import { Location } from '@angular/common';
import { Component, Directive, Injectable, Input } from '@angular/core';
import { IQueryState, ItemTypeService, commonReducers } from '@farmmaps/common';
import { Action, Store } from '@ngrx/store';
import { Feature } from 'ol';
import { Geometry } from 'ol/geom';
import { tassign } from 'tassign';
import * as mapActions from '../../actions/map.actions';
import * as mapReducers from '../../reducers/map.reducer';


@Injectable()
@Directive()
export abstract class AbstractFeatureListComponent {
  @Input() features: Array<Feature<Geometry>>;
  @Input() queryState: IQueryState;
  @Input() selectedFeature: Feature<Geometry>;
  constructor(public store: Store<mapReducers.State | commonReducers.State>, public itemTypeService: ItemTypeService, private location: Location) {
  }

  handleFeatureClick(feature:Feature<Geometry>) {
    if(feature) {
      this.store.dispatch(new mapActions.ClickFeature(feature));
      const action = this.getAction(feature);
      this.store.dispatch(action);
    }
  }

  getAction(feature:Feature<Geometry>):Action {
      const newQuery: any = tassign(mapReducers.initialState.queryState);
      newQuery.parentCode = feature.get('parentCode');
      newQuery.itemCode = feature.get('code');
      newQuery.itemType = feature.get('itemType');
      return new mapActions.DoQuery(newQuery)  
  }

  handleFeatureMouseEnter(feature:Feature<Geometry>) {
    this.store.dispatch(new mapActions.SelectFeature(feature));
  }

  handleFeatureMouseLeave() {
    this.store.dispatch(new mapActions.SelectFeature(null));
  }

  handleBackClick(event: MouseEvent) {
    event.preventDefault();
    this.location.back();
  }

  isFeatureSelected(feature:Feature<Geometry>):boolean {
    if(!this.selectedFeature) return false;
    return feature.getId() == this.selectedFeature.getId();
  }  
}

@Injectable()
@Component({
  selector: 'fm-map-feature-list',
  templateUrl: './feature-list.component.html',
  styleUrls: ['./feature-list.component.scss']
})
export class FeatureListComponent extends AbstractFeatureListComponent {

  constructor(store: Store<mapReducers.State | commonReducers.State>, itemTypeService: ItemTypeService,location:Location) {
    super(store, itemTypeService,location);
  }
}
